-- STEP 1: generate the list of users to migrate

drop table if exists users_to_merge;
create table users_to_merge (
    userIdToRemove integer,
    userIdToModify integer
);

-- This select statement will remove all USER_NAMEs where there is a corresponding USER_NAME_old1 user.
-- Comment it out if you'd prefer to manually choose users.
insert into users_to_merge (userIdToRemove, userIdToModify)
select
	new_user.cru_user_id,
  old_user.cru_user_id
from cru_user old_user
inner join cru_user new_user
	on new_user.cru_user_name = substring(old_user.cru_user_name from 1 for char_length(old_user.cru_user_name) - 5)
where old_user.cru_user_name like '%_old1';

-- This template will let you run the merge function on a specific pair of users.
-- Uncomment it if you'd prefer to individually choose pairs of users to revert.

-- insert into users_to_merge (userIdToRemove, userIdToModify)
-- select
-- 	new_user.cru_user_id,
--   old_user.cru_user_id
-- from cru_user old_user
-- cross join cru_user new_user
-- where old_user.cru_user_name = 'OLD_USER_NAME - to be modified'
-- and new_user.cru_user_name = 'NEW_USER_NAME - to be removed';





-- STEP 2: run this to do the migration

update cru_changeset_comment, users_to_merge
set cru_user_id = userIdToModify
where cru_user_id = userIdToRemove;

--

create temporary table unique_allowed_reviewers_to_update (
    userIdToRemove int,
    userIdToModify int,
    project_id int
);

insert into unique_allowed_reviewers_to_update (userIdToRemove, userIdToModify, project_id)
select
    mergePair.userIdToRemove,
    mergePair.userIdToModify,
    toRemove.cru_project_id
from cru_proj_allowed_reviewer as toRemove
inner join users_to_merge as mergePair
    on toRemove.cru_user_id = mergePair.userIdToRemove
where not exists (
    select 1 from cru_proj_allowed_reviewer as toModify
    where toModify.cru_project_id = toRemove.cru_project_id
    and toModify.cru_user_id = mergePair.userIdToModify
); 

update cru_proj_allowed_reviewer as toUpdate, unique_allowed_reviewers_to_update as ref
set cru_user_id = ref.userIdToModify
where cru_user_id = ref.userIdToRemove
and cru_project_id = ref.project_id;

drop table unique_allowed_reviewers_to_update;

delete cru_proj_allowed_reviewer
from cru_proj_allowed_reviewer
inner join users_to_merge
    on cru_user_id = users_to_merge.userIdToRemove;

--

create temporary table unique_default_reviewers_to_update (
    userIdToRemove int,
    userIdToModify int,
    project_id int
);

insert into unique_default_reviewers_to_update (userIdToRemove, userIdToModify, project_id)
select
    mergePair.userIdToRemove,
    mergePair.userIdToModify,
    toRemove.cru_project_id
from cru_proj_default_reviewer as toRemove
inner join users_to_merge as mergePair
    on toRemove.cru_user_id = mergePair.userIdToRemove
where not exists (
    select 1 from cru_proj_default_reviewer as toModify
    where toModify.cru_project_id = toRemove.cru_project_id
    and toModify.cru_user_id = mergePair.userIdToModify
); 

update cru_proj_default_reviewer as toUpdate, unique_default_reviewers_to_update as ref
set cru_user_id = ref.userIdToModify
where cru_user_id = ref.userIdToRemove
and cru_project_id = ref.project_id;

drop table unique_default_reviewers_to_update;

delete cru_proj_default_reviewer
from cru_proj_default_reviewer
inner join users_to_merge
    on cru_user_id = users_to_merge.userIdToRemove;

--

update cru_notification, users_to_merge
set cru_actioner = users_to_merge.userIdToModify
where cru_actioner = users_to_merge.userIdToRemove;

--

create temporary table unique_participant_to_update (
    userIdToRemove int,
    userIdToModify int,
    review_id int
);

insert into unique_participant_to_update (userIdToRemove, userIdToModify, review_id)
select
    mergePair.userIdToRemove,
    mergePair.userIdToModify,
    toRemove.cru_review_id
from cru_review_participant as toRemove
inner join users_to_merge as mergePair
    on toRemove.cru_user = mergePair.userIdToRemove
where not exists (
    select 1 from cru_review_participant as toModify
    where toModify.cru_review_id = toRemove.cru_review_id
    and toModify.cru_user = mergePair.userIdToModify
); 

update cru_review_participant as toUpdate, unique_participant_to_update as ref
set cru_user = ref.userIdToModify
where cru_user = ref.userIdToRemove
and cru_review_id = ref.review_id;

drop table unique_participant_to_update;

delete cru_file_read_status
from cru_file_read_status
inner join cru_review_participant partic
    on cru_file_read_status.cru_participant = partic.cru_participant_id
inner join users_to_merge
    on partic.cru_user = users_to_merge.userIdToRemove;

delete cru_review_participant
from cru_review_participant
inner join users_to_merge
    on cru_user = users_to_merge.userIdToRemove;

--

update cru_review, users_to_merge
set cru_creator = userIdToModify
where cru_creator = userIdToRemove;

--

update cru_upload_item, users_to_merge
set cru_user = userIdToModify
where cru_user = userIdToRemove;

--

update cru_recipient, users_to_merge
set cru_user = userIdToModify
where cru_user = userIdToRemove;

--

update cru_project, users_to_merge
set cru_default_moderator = userIdToModify
where cru_default_moderator = userIdToRemove;

--

create temporary table unique_comments_read_to_update (
    userIdToRemove int,
    userIdToModify int,
    comment_id int
);

insert into unique_comments_read_to_update (userIdToRemove, userIdToModify, comment_id)
select
    mergePair.userIdToRemove,
    mergePair.userIdToModify,
    toRemove.cru_comment
from cru_comment_read_status as toRemove
inner join users_to_merge as mergePair
    on toRemove.cru_user = mergePair.userIdToRemove
where not exists (
    select 1 from cru_comment_read_status as toModify
    where toModify.cru_comment = toRemove.cru_comment
    and toModify.cru_user = mergePair.userIdToModify
); 

update cru_comment_read_status as toUpdate, unique_comments_read_to_update as ref
set cru_user = ref.userIdToModify
where cru_user = ref.userIdToRemove
and cru_comment = ref.comment_id;

drop table unique_comments_read_to_update;

delete cru_comment_read_status
from cru_comment_read_status
inner join users_to_merge
    on cru_user = users_to_merge.userIdToRemove;

--

create temporary table unique_applink_to_update (
    userIdToRemove int,
    userIdToModify int,
    applink_id int
);

insert into unique_applink_to_update (userIdToRemove, userIdToModify, applink_id)
select
    mergePair.userIdToRemove,
    mergePair.userIdToModify,
    toRemove.cru_app_link_id
from cru_user_app_link as toRemove
inner join users_to_merge as mergePair
    on toRemove.cru_user = mergePair.userIdToRemove
where not exists (
    select 1 from cru_user_app_link as toModify
    where toModify.cru_app_link_id = toRemove.cru_app_link_id
    and toModify.cru_user = mergePair.userIdToModify
); 

update cru_user_app_link as toUpdate, unique_applink_to_update as ref
set cru_user = ref.userIdToModify
where cru_user = ref.userIdToRemove
and cru_app_link_id = ref.applink_id;

drop table unique_applink_to_update;

delete cru_user_app_link
from cru_user_app_link
inner join users_to_merge
    on cru_user = users_to_merge.userIdToRemove;

--

update cru_comment, users_to_merge
set cru_user_id = userIdToModify
where cru_user_id = userIdToRemove;

--

update cru_state_change, users_to_merge
set cru_user = userIdToModify
where cru_user = userIdToRemove;

--

update cru_logitem, users_to_merge
set cru_user_id = userIdToModify
where cru_user_id = userIdToRemove;

update cru_logitem, users_to_merge
set cru_entity_id = userIdToModify
where cru_entity_id = userIdToRemove
	AND cru_action in ('REVIEW_REVIEWER_ADDED', 'REVIEW_REVIEWER_ADDED_MID_REVIEW');

--

delete cru_user_profile
from cru_user_profile
inner join users_to_merge
    on cru_user_id = userIdToRemove;

delete cru_user
from cru_user
inner join users_to_merge
    on cru_user_id = userIdToRemove;

drop table users_to_merge;
